import os
import shutil
import time
from win32com.client import DispatchEx


from p2w4n.word import appendText, appendJPGFile, append

def pptx2word(file, max_slides=20, keep_images=False):
    file_name = os.path.splitext(file)[0]
    print(f'pptx2word:{file_name}')
    img_temp_folder = file_name + '_images'
    exported_word_folder = file_name + '_exported'
    word_file_prefix = file_name + '_4notion'

    # 打开 PowerPoint App


    # ppt = Dispatch('PowerPoint.Application')
    # 或者使用下面的方法，使用启动独立的进程：
    ppt = DispatchEx('PowerPoint.Application')

    # 如果不声明以下属性，运行的时候会显示的打开word
    ppt.Visible = 1  # 后台运行
    ppt.DisplayAlerts = 0  # 不显示，不警告

    try:
        pptSel = ppt.Presentations.Open(os.getcwd() + "\\" + file)
    except Exception as e:
        print(f'Open PPTX {file} failed: {e}')
        return

    if not os.path.exists(img_temp_folder):
        os.makedirs(img_temp_folder)
    
    if not os.path.exists(exported_word_folder):
        os.makedirs(exported_word_folder)
    
    
    # 打开 Word App
    word = DispatchEx('Word.Application')
    word.Visible = 0  # 后台运行 0，测试改为 1
    word.DisplayAlerts = 0  # 不显示，不警告

    # extract page to picture
    pptSel.SaveAs(os.getcwd() + "\\" + img_temp_folder, 17)
 

    # 主循环 
    tempDoc = word.Documents.Add()
    index = 0
    for slide in pptSel.Slides:
        index += 1
        print(f'slide：{index}')

        # add title of slide 
        appendText(tempDoc, f'\r\n>Slide: {index}')

        # copy slide
        # slide.Copy()
        # appendBMP(tempDoc)
        jpg = os.getcwd() + f"\\{img_temp_folder}\\Slide{index}.jpg"
        appendJPGFile(tempDoc, jpg)

        # copy notes
        tr = None
        if slide.NotesPage is not None:
            if slide.NotesPage.Shapes.Placeholders.Count > 1:
                tr = slide.NotesPage.Shapes.Placeholders(2).TextFrame.TextRange
        if not tr is None:
            txt = tr.Text
            if len(txt) > 0:
                tr.Copy()
                append(tempDoc)
        # time.sleep(0.1)

        if index % max_slides == 0:
            print(f'max_slide:{max_slides} reached index={index}, save file to ')
            tempDoc.SaveAs(os.getcwd() + f"\\{exported_word_folder}\\{word_file_prefix}-{index // max_slides}.docx"  )
            tempDoc.Close()
            tempDoc = word.Documents.Add()
    # save the last word file        
    tempDoc.SaveAs(os.getcwd() + f"\\{exported_word_folder}\\{word_file_prefix}-{index // max_slides + 1}.docx"  )
    tempDoc.Close()

    # clean up
    word.Quit()
    pptSel.Close()
    ppt.Quit()

    if not keep_images:
        shutil.rmtree(img_temp_folder)


import os
import shutil
import fitz
from win32com.client import DispatchEx

from p2w4n.word import appendText, appendJPGFile

def pdf2word(file, max_pages=20, keep_images=False):
    file_name = os.path.splitext(file)[0]
    print(f'pdf2word:{file_name}')
    img_temp_folder = file_name + '_images'
    exported_word_folder = file_name + '_exported'
    word_file_prefix = file_name + '_4notion'

    try:
        doc=fitz.open(file)
    except Exception as e:
        print(f'Open File {file} failed: {e}')
        return
    # doc = fitz.open(os.getcwd()+'\\'+file)
    if not os.path.exists(img_temp_folder):
        os.makedirs(img_temp_folder)
    
    if not os.path.exists(exported_word_folder):
        os.makedirs(exported_word_folder)
    
    
    # 打开 Word App
    word = DispatchEx('Word.Application')
    word.Visible = 0  # 后台运行 0，测试改为 1
    word.DisplayAlerts = 0  # 不显示，不警告

    # extract page to picture
    for index in range(0, doc.page_count):
        page = doc.load_page(index)
        jpg = page.get_pixmap()
        jpg.save(img_temp_folder + f'\\page-{index}.jpg')
    tempDoc = word.Documents.Add()

    # append it to a word doc
    for index in range(0, doc.page_count):
        print(f'page: {index}')

        # add title of slide 
        appendText(tempDoc, f'\r\n>Page: {index}')

        jpg = os.getcwd() + f"\\{img_temp_folder}\\page-{index}.jpg"
        appendJPGFile(tempDoc, jpg)
        if (index+1) % max_pages == 0:
            fname = os.getcwd() + f"\\{exported_word_folder}\\{word_file_prefix}-{(index+1) // max_pages}.docx"
            tempDoc.SaveAs(fname)
            print(f'max_pages[{max_pages}] reached index={index},, Save to a file: {fname}')
            tempDoc.Close()
            tempDoc = word.Documents.Add()

    # save the last word document
    # fname = os.getcwd() + f"\\{word_file_prefix}-{((index+1) // max_pages)+1}.docx" 
    fname = os.getcwd() + f"\\{exported_word_folder}\\{word_file_prefix}-{(index+1) // max_pages}.docx"
    print(f'index={index}, Final file save to {fname}')
    tempDoc.SaveAs( fname) 
    tempDoc.Close()
    word.Quit()
    if not keep_images:
        shutil.rmtree(img_temp_folder)


# 在文档末尾 粘贴
def append(doc):
    docRange = doc.Range()
    docRange.Collapse(0) # wdCollapseEnd	0; wdCollapseStart	1
    try: 
        docRange.Paste() 
    except Exception as e:
        print(f'append PPT notes to Word failed: {e}')
        return
def appendText(doc, text):
    docRange = doc.Range()
    docRange.InsertAfter(text)
def appendJPGFile(doc, jpg):
    appendText(doc, "\r\n\t")
    docRange = doc.Range()
    docRange.Collapse(0)
    docRange.InlineShapes.AddPicture(jpg)
    appendText(doc, "\r\n")


# from pdf2word import pdf2word
# from pptx2word import pptx2word
# from __init__ import __version__

from p2w4n.pdf2word import pdf2word
from p2w4n.pptx2word import pptx2word
from p2w4n import __version__
import getopt, sys
import os
def list_files_recursive():
    base = os.getcwd()
    # print(f'base={base}')
    for root, ds, fs in os.walk(base):
        for f in fs:
            suffix = os.path.splitext(f)[-1]
            # print(suffix)
            if suffix.lower() in ['.ppt', '.pptx', '.pdf']:
                
            # if f.endswith('.ppt') or f.endswith('.pptx') or f.endswith('.pdf'):
                # yield f
                fullname = os.path.join(root, f)
                # print(fullname)
                yield fullname
    # return []
def convert(f, max_pages, keep_images=False):
    suffix = os.path.splitext(f)[-1]
            # print(suffix)
    if suffix.lower() in ['.ppt', '.pptx']:
        pptx2word(f, max_pages)
    elif suffix.lower() in ['.pdf']:
        pdf2word(f, max_pages)
                
def list_files():
    base = os.getcwd()
    fs = [f for f in os.listdir(base)]
    for f in fs:
            suffix = os.path.splitext(f)[-1]
            # print(suffix)
            if suffix.lower() in ['.ppt', '.pptx', '.pdf']:
                
            # if f.endswith('.ppt') or f.endswith('.pptx') or f.endswith('.pdf'):
                yield f
                # fullname = os.path.join(root, f)
                # print(fullname)
                # yield fullname


def main():
    # 如果没有输入文件名或任何参数，就遍历当前目录，列出所有的以.ppt(x)或.pdf后缀的文件
    # 询问是否转换全部？
    # 如果不是转换全部文件，依次询问是否转换每个以.ppt(x)或.pdf后缀的文件
    # 最后问是否保留中间的jpg文件？

    print(f'p2w4n PPT/PDF转Word文档（为导入Notion）工具 - 版本 {__version__}')
    argumentList=sys.argv[1:]
    # Options
    options = "khc:p:"
    
    # Long options
    long_options = ["keep-image", "help", "convert=","pages=" ]
    
    try:
        # simulation_path = None
        # class_xlsx = None
        # Parsing argument
        arguments, values = getopt.getopt(argumentList, options, long_options)
        
        # checking each argument
        file_list = []
        source_file = None
        max_pages = 20
        keep_images = False

        for currentArgument, currentValue in arguments:
            print(f'currentArgument:{currentArgument}=[{currentValue}]')
            if currentArgument in ("-h", "--help"):
                print ("\np2w4n <pdf或ppt文件名>")
                return
                
            elif currentArgument in ("-c", "--convert"):
                print ("转换文件: %s" % (currentValue))
                source_file = currentValue
            elif currentArgument in ("-k", "--keep-images"):
                print ("保留中间图片文件")
                keep_images = True
                
            elif currentArgument in ("-p", "--pages"):
                print (("每个word文件最大页面数： %s") % (currentValue))
                max_pages = currentValue
        if source_file is None:
            # 遍历当前目录，列出所有的以.ppt(x)或.pdf后缀的文件
            print('目录下全部PPT或PDF文件：')
            file_list = list_files()
        else:
            # file_list.append(source_file)
            convert(source_file, max_pages, keep_images)
            return
        # print(f'Files: {file_list}')
        index = 1
        convert_list = []
        for f in file_list:
            print(f'{index}.\t{f}')
            index += 1
            convert_list.append(f)
        action = input(f'[A] (全部转换)\n[S] (让我选择)\n您的选择是:')
        todo = 0
        if action.lower() == 'a' or action.lower() == 'all':
            todo = 1
        elif action.lower() == 's' or action.lower() == 'select':
            todo = 2

        if todo == 0:
            print("您没有选择任何文件转换，退出。")
            return
        for file in convert_list:
            if todo == 1:
                convert(file, max_pages, keep_images)
            else:
                to_convert = input(f'转换 [{file}] ? [Y/N] ')
                if to_convert.lower() == 'y' or to_convert.lower() == 'yes':
                    convert(file, max_pages, keep_images)

    except getopt.error as err:
        # output error, and return with an error code
        print (str(err))

if __name__ == "__main__":
    print(f'\n{sys.argv[0]}')
    main()


from setuptools import setup, find_packages
import p2w4n

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup (
    name="p2w4n",
    version=p2w4n.__version__,
    author="Learner Chen",
    author_email="learner.chen@icloud.com",
    description="Convert PDF/PPT(X) file word document readfy for import into Notion.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="NO-DISTRIBUTION",
    url="https://www.3thinking.cn/p2w4n/",
    # packages=find_packages(),
    packages=['p2w4n'],
    package_dir={'p2w4n': 'p2w4n'},
    entry_points={'console_scripts': ['p2w4n = p2w4n.__main__:main']},
    include_package_data=True,
    install_requires=[
        "PyMuPDF==1.20.2",
        "pywin32==304"        
    ],
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7"
    ]
)